"""source: https://gist.github.com/enamoria/fa9baa906f23d1636c002e7186516a7b
"""
import pandas as pd


def memory_usage(df: pd.DataFrame) -> str:
    """A helper function that evaluates memory in a DataFrame.

    :param df: A Pandas DataFrame
    :return: An f-string that indicates DataFrame memory usage in MB.
    """
    mem = df.memory_usage().sum() / 1024 ** 2

    return '{:.2f} MB'.format(mem)



"""Base class for pipeline stages
"""
import abc
import logging


class BaseStage(metaclass=abc.ABCMeta):
    """Base class for stage in the pipeline / workflow
    """
    logger = logging.getLogger("pipeline").getChild("base_stage")
    name = "base"

    def __init__(self, parent=None):
        """The init function.
        Args:
            parent: parent stage.
        """
        self.parent = parent

    def execute(self):
        """The function that is called from the outside to execute the stage.
        Returns:
            True if the stage execution succeeded, False otherwise.
        """
        self.pre_run()
        if self.run():
            self.post_run()
            return True
        else:
            self.on_failure()
            return False

    def pre_run(self):
        """The function that is executed before the stage is run.
        """

    @abc.abstractmethod
    def run(self):
        """Runs the stage.
        Returns:
            True if the stage execution succeeded, False otherwise.
        """

    def post_run(self):
        """The function that is executed after the stage is run successfully.
        """
        self.logger.info("-" * 40)
        self.logger.info("=" * 40)

    def on_failure(self):
        """The function that is executed after the stage run failed.
        """


import pandas as pd
import re


def normalize_column_names(df: pd.DataFrame) -> pd.DataFrame:
    """A helper function to standardize column names.

    :param df: A Pandas DataFrame.
    :return: A DataFrame with lowercase column names and spaces as underscore separators.
    """
    df.columns = [re.sub(r'[^\w\s]', ' ', i) for i in df.columns]
    cols = list(map(str.split, df.columns))
    cols = [' '.join(i) for i in cols]
    df.columns = cols
    df.columns = map(str.strip, df.columns)

    df.columns = map(str.lower, df.columns)
    df.columns = df.columns.str.replace(' ', '_')
    df.columns = df.columns.str.replace('-', '_')
    return df



"""A generic base factory class
"""


class BaseFactory:
    """A base factory class.
    """

    # Internal registry for keeping track of available things to build.
    registry = {}

    @classmethod
    def register(cls, name):
        """Adds cls to the registry under name.

        Args:
            name: name of the model class.
        """
        def inner_wrapper(new_class):
            cls.registry[name] = new_class
            return new_class

        return inner_wrapper

    @classmethod
    def create(cls, name, **kwargs):
        """Factory command to build.

        Args:
            name: the name of the widget to build.
            kwargs: the configuration parameters for the build.

        Returns:
        """
        if name not in cls.registry:
            raise LookupError("\"{}\" not found in registry.".format(name))

        to_del = []
        config = {}
        for key in kwargs:
            if key.endswith("config"):
                if not kwargs[key] is None:
                    config.update(kwargs[key])
                to_del.append(key)

        for key in to_del:
            del kwargs[key]

        return cls.registry[name](**kwargs, **config)



import logging
import sys

from datetime import datetime
from os.path import join

import os

from typing import Optional, NoReturn


def configure_logger(log_dir: Optional[str] = os.getcwd()) -> NoReturn:
    """Runs basic configuration for the workflow.
    """
    logger = logging.getLogger("pipeline").getChild("configuration")
    format = "%(asctime)s:%(name)s:%(levelname)s:%(message)s"

    timestamp = datetime.now().strftime("%Y%m%d_%H%M")
    log_filename = "{}.log".format(timestamp)
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    filename = join(log_dir, log_filename)

    log_formatter = logging.Formatter(format)
    stream_handler = logging.StreamHandler(sys.stdout)
    stream_handler.setFormatter(log_formatter)

    logging.basicConfig(filename=filename, format=format, level=logging.INFO)
    logging.getLogger().addHandler(stream_handler)

    logger.info("Logging configurations finished.")



from numpy import (float16, float32, float64)
from numpy import finfo

from typing import Dict


def np_float_scalars() -> Dict:
    """A convenience function to define data structures and ranges of common numpy float scalars.

    :return: A dictionary of numpy floating point sclars and their associated ranges
    """
    scalars = [float16, float32, float64]

    return {scalar: (finfo(scalar).min, finfo(scalar).max + 1.) for scalar in scalars}



from numpy import (int8, int16, int32, int64,
                   uint8, uint16, uint32, uint64,
                   )
from numpy import iinfo

from typing import Dict


def np_int_scalars() -> Dict:
    """A convenience function to define data structures and ranges of common numpy integer scalars.

    :return: A dictionary of numpy floating point sclars and their associated ranges
    """
    scalars = [int8, int16, int32, int64,
               uint8, uint16, uint32, uint64]

    return {scalar: range(iinfo(scalar).min, iinfo(scalar).max + 1) for scalar in scalars}



"""Black Magic decorators.
"""
import functools


def singleton(cls):
    """Make a class a Singleton class (only one instance).
    Args:
        cls: class to make a singleton.
    Returns:
        decorator wrapper class.
    """
    @functools.wraps(cls)
    def wrapper_singleton(*args, **kwargs):
        if not wrapper_singleton.instance:
            wrapper_singleton.instance = cls(*args, **kwargs)
        return wrapper_singleton.instance

    wrapper_singleton.instance = None
    return wrapper_singleton


def signature(name=None, variable_names=None):
    """Add signature function to the class.
    Args:
        name: name to include in the signature.
        variable_names: names of the variables to include in the signature.
    Returns:
        decorator wrapper function.
    """
    def decorator(cls):
        """Add signature function to the class.
        Args:
            cls: the class to modify.
        Returns:
            modified class.
        """
        nonlocal name, variable_names

        if not variable_names:
            variable_names = []
        if not name:
            name = cls.__name__

        def _signature(self):
            """Signature function.
            Returns:
                A string that represents the instance.
            """
            values = [name] + [str(self.__dict__[var_name]) for var_name in variable_names]
            return "_".join(values)

        setattr(cls, "signature", _signature)

        return cls

    return decorator



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='sailfish',
    version='0.0.1',
    author='Justin Chae',
    author_email='justin.chae@dyna-lab.com',
    description='Sailfish provides helpful utilities and pipelines to optimize Pandas dataframes. This Sailfish project'
                'replaces pd-helper 1.0.0 which is deprecated with sailfish 1.0.0.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/justinhchae/sailfish",
    project_urls={
        "Download URL": "https://github.com/justinhchae/sailfish/archive/refs/tags/0.0.2.tar.gz",
    },
    classifiers=[
        # "Development Status :: 5 - Production/Stable",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Information Analysis',
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    install_requires=['pandas', 'numpy', 'tqdm', 'shortuuid'],
    python_requires=">=3.7",
)
